-----------------------------------------------------------------------------
-- |
-- Module      :  MkCore
-- Project     :  HBC
-- Copyright   :  (c) Hal Daume III
-- License     :  Open
-- 
-- Maintainer  :  Hal Daume III <me@hal3.name>
-- Stability   :  stable
-- Portability :  portable
--
-- Perform all operations necessary to convert DECL to Core
--
-----------------------------------------------------------------------------

module MkCore where -- (mkCore, coreToPrior) where

import Decl
import Core
import Util
import Blanket
import Reindex
import Lift
import Translate
import Data.Maybe
import Debug.Trace

mkCore :: MOD -> Core
mkCore mod 
--  | trace (show lMod) False = undefined
--  | trace (show blankets) False = undefined
--  | trace (show reInd) False = undefined
  | otherwise = fiddleSP $ Core (zipWith3 mkTranslation decls reInd ranges)
  where
    -- first, perform lifting
    lMod@(MOD decls) = lift mod
    nn = length decls

    -- second, identify markov blankets
    blankets = map (blanket lMod) [0..(nn-1)]

    -- third, reindex all blankets
    reInd = zipWith (\l j -> map (\ (i,e) -> reindex (decls!!j) (decls!!i) e) l) blankets [0..]

    -- extract ranges
    ranges = [ranges | DECL _ _ ranges <- decls]
--zipWith (\d (i,e) -> map (reindex d.snd)) decls blankets

coreToPrior :: Core -> Core
coreToPrior core = Core (mapMaybe coreToPrior' dl)
  where
    Core dl = unliftCore core

    coreToPrior' (var, dist, rl) =
      case findPriorDist var dist of
        [] -> Nothing
        l  -> Just (var, foldr1 (:*:) l, rl)
    findPriorDist var d@(Dist v _ _) 
      | v == var  = [d]
      | otherwise = []
    findPriorDist var (a :*: b)  = findPriorDist var a ++ findPriorDist var b
    findPriorDist var (a :^: _)  = findPriorDist var a
    findPriorDist var (Prod _ d _) = findPriorDist var d

{-
lMod@(MOD decls) = lift TestLDA.ldaMOD
nn = length decls
blankets = map (blanket lMod) [0..(nn-1)]
reInd = zipWith (\l j -> map (\ (i,e) -> reindex (decls!!j) (decls!!i) e) l) blankets [0..]
-}

mkTranslation :: DECL -> [((VAR, [EXPR]), RHS, [RANGE])] -> [RANGE] -> (Var, Dist, [Range])
mkTranslation (DECL (LHS v0 ind0) (RHS dist e lift) ranges) rhsL range0 =
  (var0, foldr (:*:) prior $ map (mkRHS ranges) rhsL, map translateRange range0)
  where
    var0  = V v0 (map (\i -> Id i Nothing) ind0)
    prior = addLift lift (Dist var0 dist $ map translateExpr e)

mkRHS :: [RANGE] -> ((VAR, [EXPR]), RHS, [RANGE]) -> Dist
mkRHS ranges0 ((v,ind), RHS dist e lift, ranges) = foldr addRange post unR
  where
    var0 = V v (map translateSExpr ind)
    post = addLift lift (Dist var0 dist $ map translateExpr e)
    unR  = filter (unqualifiedRanges ranges0) ranges

addLift :: LIFT -> Dist -> Dist
addLift lift d = foldr addLiftProd (foldr addLiftExp d lift) lift
  where
    addLiftProd (v,_) d = Prod (unkRange v) d ""
    addLiftExp  (v,e) d = d :^: (Func "=" [translateExpr v, translateExpr e])

--    addLift' (v,e) d = 
--      Prod (unkRange v) (d :^: (Func "=" [translateExpr v, translateExpr e])) ""
    unkRange v = Range (head $ exprVars v) unkV unkV

unqualifiedRanges :: [RANGE] -> RANGE -> Bool
unqualifiedRanges l r = not (rangeLHS r `elem` map rangeLHS l)
  where rangeLHS (RANGE v _ _) = v

addRange :: RANGE -> Dist -> Dist
addRange r d = Prod (translateRange r) d ""

